@@ -23,17 +23,8 @@
 #' @param par_link An optional logical vector indicating whether the logarithmic/logistic link should be applied to restricted parameters in order to obtain unrestricted values. Defaults to applying the logarithmic/logistic link for time-varying parameters and keeping the original link for constant parameters.
 #' @param par_init An optional numeric vector of initial values of time-varying parameters. For \code{NA} values or when \code{NULL}, set initial values to unconditional values of time-varying parameters. For example, in the case of GAS(1,1) model with \code{regress = "joint"}, to \code{omega / (1 - phi1)}. Not to be confused with starting values for the optimization \code{coef_start}.
 #' @param lik_skip A numeric value specifying the number of skipped observations at the beginning of the time series or after \code{NA} values in the likelihood computation. Defaults to \code{0L}, i.e. the full likelihood. If \code{NULL}, it is selected as \code{max(p,q)}, i.e. the conditional likelihood.
-#' @param coef_fix_value An optional numeric vector of values to which coefficients are to be fixed. \code{NA} values represent coefficients to be estimated.
-#' @param coef_fix_other An optional square numeric matrix of multiples of the estimated coefficients, which are to be added to the fixed coefficients. This allows the fixed coefficients to be linear combinations of the estimated coefficients. A coefficient given by row is fixed on coefficient given by column. By this logic, all rows corresponding to the estimated coefficients should contain only \code{NA} values. Furthermore, all columns corresponding to the fixed coefficients should also contain only \code{NA} values.
-#' @param coef_fix_special An optional character vector of predefined structures of \code{coef_fix_value} and \code{coef_fix_other}. Useful mainly for multidimensional models. Value \code{"panel_structure"} forces all regression, autoregression, and score coefficients to be the same for all time-varying parameters within their group. Value \code{"zero_sum_intercept"} forces all constant parameters to sum up to zero within their group. Value \code{"random_walk"} forces all autoregressive coefficients to be equal to one (should be used with caution due to nonstationarity; \code{par_init} must be specified). Multiple predefined structures can be used together. Also can be used in combination with custom \code{coef_fix_value} and \code{coef_fix_other}.
-#' @param coef_bound_lower An optional numeric vector of lower bounds on coefficients.
-#' @param coef_bound_upper An optional numeric vector of upper bounds on coefficients.
-#' @param coef_start An optional numeric vector of starting values for the optimization. If not supplied, starting values are selected from a small grid of values.
-#' @param optim_function An optimization function. For suitable wrappers of common R optimization functions, see \code{\link[gasmodel:wrappers_optim]{wrappers_optim}}. Can be set to \code{NULL} if the optimal solution should not be computed, which can be useful if the goal is only to evaluate the fit for the coefficients specified in argument \code{coef_start}.
-#' @param optim_arguments An optional list of arguments to be passed to the optimization function.
-#' @param hessian_function A Hessian function. For suitable wrappers of common R Hessian functions, see \code{\link[gasmodel:wrappers_hessian]{wrappers_hessian}}. Can be set to \code{NULL} if the Hessian matrix should not be computed, which can speed up computations when asymptotic inference is not desired.
-#' @param hessian_arguments An optional list of arguments to be passed to the Hessian function.
-#' @param print_progress A logical value indicating whether to progressively print a detailed report on computation.
+#' @param coef_fix_value,coef_fix_other,coef_fix_special,coef_bound_lower,coef_bound_upper Restrictions on the coefficients, see the section \emph{Restricting the Coefficients} below for more details.
+#' @param coef_start,optim_function,optim_arguments,hessian_function,hessian_arguments,print_progress Controls for numerical procedures; see the section \emph{Controlling Numerical Procedures} below for more details.
 #'
 #' @details
 #' The generalized autoregressive score (GAS) models of Creal et al. (2013) and Harvey (2013), also known as dynamic conditional score (DCS) models or score-driven (SD) models, have established themselves as a useful modern framework for time series modeling.
@@ -60,11 +51,31 @@
 #' For a comparison of the GAS models to alternative models, see Koopman et al. (2016) and Blazsek and Licht (2020).
 #'
 #' The GAS class includes many well-known econometric models, such as the generalized autoregressive conditional heteroskedasticity (GARCH) model of Bollerslev (1986), the autoregressive conditional duration (ACD) model of Engle and Russel (1998), and the Poisson count model of Davis et al. (2003).
-#' More recently, a variety of novel score-driven models has been proposed, such as the Beta-t-(E)GARCH model of Harvey and Chakravarty (2008), the discrete price changes model of Koopman et al. (2018), the circular model of Harvey (2019), the bivariate Poisson model of Koopman and Lit (2019), and the ranking model of Holý and Zouhar (2022).
+#' More recently, a variety of novel score-driven models has been proposed, such as the Beta-t-(E)GARCH model of Harvey and Chakravarty (2008), the discrete price changes model of Koopman et al. (2018), the circular model of Harvey et al. (2024), the bivariate Poisson model of Koopman and Lit (2019), and the ranking model of Holý and Zouhar (2022).
 #' For an overview of various GAS models, see Harvey (2022).
 #'
 #' The extensive GAS literature is listed on \href{https://www.gasmodel.com}{www.gasmodel.com}.
 #'
+#' @section Restricting the Coefficients:
+#'
+#' The coefficients can be restricted in several ways during estimation. They can be set to specific values using a numeric vector supplied via \code{coef_fix_value}. \code{NA} values indicate coefficients to be estimated.
+#'
+#' Furthermore, the fixed coefficients can be expressed as linear combinations of the estimated coefficients using an optional square numeric matrix of multiples of the estimated coefficients, supplied via \code{coef_fix_other}. A coefficient given by a row is fixed on the coefficient given by a column. By this logic, all rows corresponding to the estimated coefficients should contain only \code{NA} values. Likewise, all columns corresponding to the fixed coefficients should also contain only \code{NA} values.
+#'
+#' For convenience, there are several predefined structures for \code{coef_fix_value} and \code{coef_fix_other}, which can be specified as a character vector supplied via \code{coef_fix_special}. This is useful mainly for multidimensional models. The value \code{"panel_structure"} forces all regression, autoregression, and score coefficients to be the same for all time-varying parameters within their group. The value \code{"zero_sum_intercept"} forces all constant parameters to sum to zero within their group. The value \code{"random_walk"} forces all autoregressive coefficients to be equal to one (should be used with caution due to nonstationarity; \code{par_init} must be specified). Multiple predefined structures can be used together and can also be combined with custom \code{coef_fix_value} and \code{coef_fix_other}.
+#'
+#' Finally, instead of fixing the coefficients, they can be bounded. Lower bounds can be set using a numeric vector supplied via \code{coef_bound_lower}, while upper bounds can be set in a smilar way via \code{coef_bound_upper}. For example, the autoregressive coefficient can be bounded above by 1.
+#'
+#' @section Controlling Numerical Procedures:
+#'
+#' Starting values for the optimization can be supplied as a numeric vector via \code{coef_start}. If not supplied, starting values are selected from a small grid of values.
+#'
+#' A custom optimizer can be supplied as a function via \code{optim_function}. For suitable wrappers of common R optimization functions, see \code{\link[gasmodel:wrappers_optim]{wrappers_optim}}. Can be set to \code{NULL} if the optimal solution should not be computed, which can be useful if the goal is only to evaluate the fit for the coefficients specified in argument \code{coef_start}. An optional list of arguments to be passed to the optimization function can be supplied via \code{optim_arguments}.
+#'
+#' A custom Hessian computation can be supplied as a function via \code{hessian_function}. For suitable wrappers of common R Hessian functions, see \code{\link[gasmodel:wrappers_hessian]{wrappers_hessian}}. Can be set to \code{NULL} if the Hessian matrix should not be computed, which can speed up computations when asymptotic inference is not desired. An optional list of arguments to be passed to the Hessian function can be supplied via \code{hessian_arguments}.
+#'
+#' Setting \code{print_progress = TRUE} enables a detailed computation report that is printed progressively.
+#'
 #' @return A \code{list} of S3 class \code{gas} with components:
 #' \item{data$y}{The time series.}
 #' \item{data$x}{The exogenous variables.}
@@ -144,7 +155,7 @@
 #'
 #' Harvey, A. C. and Chakravarty, T. (2008). Beta-t-(E)GARCH. \emph{Cambridge Working Papers in Economics}, CWPE 0840. \doi{10.17863/cam.5286}.
 #'
-#' Harvey, A., Hurn, S., and Thiele, S. (2019). Modeling Directional (Circular) Time Series. \emph{Cambridge Working Papers in Economics}, CWPE 1971. \doi{10.17863/cam.43915}.
+#' Harvey, A., Hurn, S., Palumbo, D., and Thiele, S. (2024). Modelling Circular Time Series. \emph{Journal of Econometrics}, \strong{239}(1), 105450. \doi{10.1016/j.jeconom.2023.02.016}.
 #'
 #' Holý, V. and Zouhar, J. (2022). Modelling Time-Varying Rankings with Autoregressive and Score-Driven Dynamics. Journal of the Royal Statistical Society: Series C (Applied Statistics), \strong{71}(5). \doi{10.1111/rssc.12584}.
 #'
@@ -192,77 +203,19 @@
 #'
 #' @export
 gas <- function(y, x = NULL, distr, param = NULL, scaling = "unit", regress = "joint", p = 1L, q = 1L, par_static = NULL, par_link = NULL, par_init = NULL, lik_skip = 0L, coef_fix_value = NULL, coef_fix_other = NULL, coef_fix_special = NULL, coef_bound_lower = NULL, coef_bound_upper = NULL, coef_start = NULL, optim_function = wrapper_optim_nloptr, optim_arguments = list(opts = list(algorithm = 'NLOPT_LN_NELDERMEAD', xtol_rel = 0, maxeval = 1e6)), hessian_function = wrapper_hessian_stats, hessian_arguments = list(), print_progress = FALSE) {
-  model <- list()
-  model$distr <- check_my_distr(distr = distr)
-  model$param <- check_my_param(param = param, distr = model$distr)
-  model$scaling <- check_my_scaling(scaling = scaling)
-  model$regress <- check_my_regress(regress = regress)
-  info_distr <- info_distribution(distr = model$distr, param = model$param)
-  data <- list()
-  data$y <- check_my_y(y = y, dim = info_distr$dim, type = info_distr$type)
-  model$t <- check_my_t(y = data$y)
-  model$n <- check_my_n(y = data$y)
-  info_par <- info_parameters(distr = model$distr, param = model$param, n = model$n)
-  data$x <- check_my_x(x = x, t = model$t, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-  model$m <- check_my_m(x = data$x)
-  model$p <- check_my_p(p = p, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-  model$q <- check_my_q(q = q, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-  model$par_static <- check_my_par_static(par_static = par_static, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-  model$par_static[model$m == 0L & model$p == 0L & model$q == 0L] <- TRUE
-  data$x[model$par_static] <- list(matrix(NA_real_, nrow = model$t, ncol = 0L))
-  model$m[model$par_static] <- 0L
-  model$p[model$par_static] <- 0L
-  model$q[model$par_static] <- 0L
-  model$par_link <- check_my_par_link(par_link = par_link, par_static = model$par_static, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-  info_par <- info_linked_parameters(info_par = info_par, par_link = model$par_link)
-  model$m <- name_vector(model$m, info_par$par_names)
-  model$p <- name_vector(model$p, info_par$par_names)
-  model$q <- name_vector(model$q, info_par$par_names)
-  model$par_static <- name_vector(model$par_static, info_par$par_names)
-  model$par_link <- name_vector(model$par_link, info_par$par_names)
-  model$par_init <- name_vector(check_my_par_init(par_init = par_init, par_num = info_par$par_num), info_par$par_names)
-  model$lik_skip <- check_my_lik_skip(lik_skip = lik_skip, t = model$t, p = model$p, q = model$q)
-  info_coef <- info_coefficients(m = model$m, p = model$p, q = model$q, par_static = model$par_static, par_names = info_par$par_names, par_num = info_par$par_num, group_names = info_par$group_names, group_of_par_names = info_par$group_of_par_names)
-  model$coef_fix_value <- check_my_coef_fix_value(coef_fix_value = coef_fix_value, coef_num = info_coef$coef_num)
-  model$coef_fix_other <- check_my_coef_fix_other(coef_fix_other = coef_fix_other, coef_fix_value = model$coef_fix_value, coef_num = info_coef$coef_num)
-  model$coef_fix_special <- check_my_coef_fix_special(coef_fix_special = coef_fix_special)
-  model[c("coef_fix_value", "coef_fix_other")] <- fixed_coefficients(model = model, info_par = info_par, info_coef = info_coef)
-  model$coef_bound_lower <- name_vector(check_my_coef_bound_lower(coef_bound_lower = coef_bound_lower, par_static = model$par_static, par_support = info_par$par_support, par_num = info_par$par_num, coef_in_par_num = info_coef$coef_in_par_num, coef_num = info_coef$coef_num), info_coef$coef_names)
-  model$coef_bound_upper <- name_vector(check_my_coef_bound_upper(coef_bound_upper = coef_bound_upper, coef_bound_lower = model$coef_bound_lower, par_static = model$par_static, par_support = info_par$par_support, par_num = info_par$par_num, coef_in_par_num = info_coef$coef_in_par_num, coef_num = info_coef$coef_num), info_coef$coef_names)
-  info_theta <- info_thetas(coef_fix_value = model$coef_fix_value, coef_fix_other = model$coef_fix_other, coef_names = info_coef$coef_names)
-  fun <- list()
-  fun$loglik <- setup_fun_loglik(distr = model$distr, param = model$param, par_trans = info_par$par_trans)
-  fun$mean <- setup_fun_mean(distr = model$distr, param = model$param, par_trans = info_par$par_trans)
-  fun$var <- setup_fun_var(distr = model$distr, param = model$param, par_trans = info_par$par_trans)
-  fun$score <- setup_fun_score(distr = model$distr, param = model$param, scaling = model$scaling, orthog = info_distr$orthog, par_trans = info_par$par_trans, par_static = model$par_static)
-  comp <- list()
-  comp$coef_start <- check_my_coef_start(coef_start = coef_start, coef_bound_lower = model$coef_bound_lower, coef_bound_upper = model$coef_bound_upper, coef_num = info_coef$coef_num)
-  comp$theta_start <- convert_coef_vector_to_theta_vector(comp$coef_start, coef_fix_value = model$coef_fix_value, coef_fix_other = model$coef_fix_other)
-  comp$theta_bound_lower <- convert_coef_vector_to_theta_vector(model$coef_bound_lower, coef_fix_value = model$coef_fix_value, coef_fix_other = model$coef_fix_other)
-  comp$theta_bound_upper <- convert_coef_vector_to_theta_vector(model$coef_bound_upper, coef_fix_value = model$coef_fix_value, coef_fix_other = model$coef_fix_other)
-  comp$compute_start <- any(is.na(comp$theta_start))
-  control <- list()
-  if (is.null(optim_function)) {
-    comp$compute_optim <- FALSE
-    control['optim_function'] <- list(NULL)
-    control['optim_arguments'] <- list(NULL)
-  } else {
-    comp$compute_optim <- TRUE
-    control$optim_function <- check_generic_function(arg = optim_function, arg_name = "optim_function")
-    control$optim_arguments <- check_generic_list(arg = optim_arguments, arg_name = "optim_arguments")
-  }
-  if (is.null(hessian_function)) {
-    comp$compute_hessian <- FALSE
-    control['hessian_function'] <- list(NULL)
-    control['hessian_arguments'] <- list(NULL)
-  } else {
-    comp$compute_hessian <- TRUE
-    control$hessian_function <- check_generic_function(arg = hessian_function, arg_name = "hessian_function")
-    control$hessian_arguments <- check_generic_list(arg = hessian_arguments, arg_name = "hessian_arguments")
-  }
-  comp$print_progress <- check_generic_logical_scalar(arg = print_progress, arg_name = "print_progress")
-  comp$est_details <- list(data = data, model = model, fun = fun, info_distr = info_distr, info_par = info_par, info_coef = info_coef, print_progress = comp$print_progress)
+  # Load auxiliary variables:
+  load <- load_estimate(y = y, x = x, distr = distr, param = param, scaling = scaling, regress = regress, p = p, q = q, par_static = par_static, par_link = par_link, par_init = par_init, lik_skip = lik_skip, coef_fix_value = coef_fix_value, coef_fix_other = coef_fix_other, coef_fix_special = coef_fix_special, coef_bound_lower = coef_bound_lower, coef_bound_upper = coef_bound_upper, coef_start = coef_start, optim_function = optim_function, optim_arguments = optim_arguments, hessian_function = hessian_function, hessian_arguments = hessian_arguments, print_progress = print_progress)
+  data <- load$data
+  model <- load$model
+  control <- load$control
+  fun <- load$fun
+  info_distr <- load$info_distr
+  info_par <- load$info_par
+  info_coef <- load$info_coef
+  info_theta <- load$info_theta
+  comp <- load$comp
   solution <- list()
+  # Compute starting solution:
   if (comp$compute_start) {
     if (comp$print_progress) { message("Computing a starting solution...") }
     comp$result_start <- starting_theta(theta_start = solution$theta_start, theta_bound_lower = comp$theta_bound_lower, theta_bound_upper = comp$theta_bound_upper, data = data, model = model, fun = fun, info_distr = info_distr, info_par = info_par, info_coef = info_coef, info_theta = info_theta, print_progress = comp$print_progress)
@@ -275,9 +228,10 @@ gas <- function(y, x = NULL, distr, param = NULL, scaling = "unit", regress = "j
     solution$status_start <- "starting_values_supplied"
     solution$theta_start <- name_vector(comp$theta_start, info_theta$theta_names)
   }
+  # Compute optimal solution:
   if (comp$compute_optim) {
     if (comp$print_progress) { message("Computing the optimal solution...") }
-    comp$result_optim <- do.call(control$optim_function, args = c(list(obj_fun = likelihood_objective, theta_start = solution$theta_start, theta_bound_lower = comp$theta_bound_lower, theta_bound_upper = comp$theta_bound_upper, est_details = comp$est_details), control$optim_arguments))
+    comp$result_optim <- be_silent_but_theta(do.call(control$optim_function, args = c(list(obj_fun = likelihood_objective, theta_start = solution$theta_start, theta_bound_lower = comp$theta_bound_lower, theta_bound_upper = comp$theta_bound_upper, est_details = comp$est_details), control$optim_arguments)))
     solution$status_optim <- comp$result_optim$status_optim
     solution$theta_optim <- name_vector(comp$result_optim$theta_optim, info_theta$theta_names)
     if (!(solution$status_optim %in% c("success", "objective_tolerance_reached", "desired_objective_reached", "variables_tolerance_reached"))) {
@@ -287,9 +241,10 @@ gas <- function(y, x = NULL, distr, param = NULL, scaling = "unit", regress = "j
     solution$status_optim <- "computation_skipped"
     solution$theta_optim <- solution$theta_start
   }
+  # Compute Hessian matrix:
   if (comp$compute_hessian) {
     if (comp$print_progress) { message("Computing the Hessian matrix...") }
-    comp$result_hessian <- do.call(control$hessian_function, args = c(list(obj_fun = likelihood_objective, theta_optim = solution$theta_optim, est_details = comp$est_details), control$hessian_arguments))
+    comp$result_hessian <- be_silent_but_theta(do.call(control$hessian_function, args = c(list(obj_fun = likelihood_objective, theta_optim = solution$theta_optim, est_details = comp$est_details), control$hessian_arguments)))
     solution$status_hessian <- comp$result_hessian$status_hessian
     solution$theta_hessian <- name_matrix(comp$result_hessian$theta_hessian, info_theta$theta_names, info_theta$theta_names)
     if (solution$status_hessian != "success") {
@@ -299,6 +254,7 @@ gas <- function(y, x = NULL, distr, param = NULL, scaling = "unit", regress = "j
     solution$status_hessian <- "computation_skipped"
     solution$theta_hessian <- name_matrix(matrix(NA_real_, nrow = info_theta$theta_num, ncol = info_theta$theta_num), info_theta$theta_names, info_theta$theta_names)
   }
+  # Compute inference:
   fit <- list()
   fit$coef_est <- name_vector(convert_theta_vector_to_coef_vector(solution$theta_optim, coef_fix_value = model$coef_fix_value, coef_fix_other = model$coef_fix_other), info_coef$coef_names)
   comp$eval_tv <- be_silent(likelihood_evaluate(coef = fit$coef_est, data = data, model = model, fun = fun, info_par = info_par, info_coef = info_coef))
@@ -315,21 +271,25 @@ gas <- function(y, x = NULL, distr, param = NULL, scaling = "unit", regress = "j
   } else if (model$regress == "sep") {
     fit$par_unc <- sapply(1:info_par$par_num, function(i) { comp$struc[[i]]$omega + colMeans(data$x[[i]], na.rm = TRUE) %*% comp$struc[[i]]$beta })
   }
+  # Format data:
   info_data <- info_data(y = data$y, x = data$x)
   data$y <- name_matrix(data$y, info_data$index_time, info_data$index_series, drop = c(FALSE, TRUE))
   data$x <- name_list_of_matrices(data$x, info_par$par_names, info_data$index_time_list, info_data$index_vars_list, drop = c(FALSE, TRUE), zero = c(FALSE, TRUE))
+  # Format time-varying quantities:
   fit$par_tv <- name_matrix(comp$eval_tv$par, info_data$index_time, info_par$par_names, drop = c(FALSE, TRUE))
   fit$score_tv <- name_matrix(comp$eval_tv$score, info_data$index_time, info_par$par_names, drop = c(FALSE, TRUE))
   fit$mean_tv <- name_matrix(fun$mean(comp$eval_tv$par), info_data$index_time, info_data$index_series, drop = c(FALSE, TRUE))
   fit$var_tv <- name_matrix(convert_varcov_array_to_var_matrix(fun$var(comp$eval_tv$par)), info_data$index_time, info_data$index_series, drop = c(FALSE, TRUE))
   fit$resid_tv <- (data$y - fit$mean_tv) / sqrt(fit$var_tv)
   fit$loglik_tv <- name_vector(comp$eval_tv$lik, info_data$index_time)
+  # Determine fit:
   fit$loglik_sum <- sum(fit$loglik_tv, na.rm = TRUE)
   fit$aic <- 2 * model$num_coef - 2 * fit$loglik_sum
   fit$bic <- log(model$num_obs) * model$num_coef - 2 * fit$loglik_sum
   if (mean(fit$loglik_tv, na.rm = TRUE) <= -1e100) {
     warning("The likelihood function has zero value. The results are not reliable.")
   }
+  # Return results:
   report <- list(data = data, model = model, control = control, solution = solution, fit = fit)
   class(report) <- "gas"
   return(report)
