@@ -12,15 +12,15 @@
 #' Method \code{"given_coefs"} computes a path of time-varying parameters for each supplied coefficient set.
 #' Instead of supplying arguments about the model, the function can be applied to the \code{gas} object obtained by the \code{\link[gasmodel:gas]{gas()}} function.
 #'
-#' @inheritParams gas
-#' @inheritParams gas_forecast
-#'
+#' @param gas_object An optional GAS estimate, i.e. a list of S3 class \code{gas} returned by function \code{\link[gasmodel:gas]{gas()}}.
 #' @param method A method used for parameter uncertainty. Supported methods are \code{"given_coefs"} and \code{"simulated_coefs"}.
 #' @param coef_set A numeric matrix of coefficient sets in rows for \code{method = "given_coefs"}. Can be generated for example by \code{\link[gasmodel:gas_bootstrap]{gas_bootstrap()}}.
 #' @param rep_gen A number of generated coefficient sets for \code{method = "simulated_coefs"}.
+#' @param t_ahead A number of observations to forecast.
+#' @param x_ahead Out-of-sample exogenous variables. For a single variable common for all time-varying parameters, a numeric vector. For multiple variables common for all time-varying parameters, a numeric matrix with observations in rows. For individual variables for each time-varying parameter, a list of numeric vectors or matrices in the above form. The number of observation must be equal to \code{t_ahead}.
 #' @param rep_ahead A number of simulation repetitions for forecasting when \code{t_ahead > 0}.
 #' @param quant A numeric vector of probabilities determining quantiles.
-#' @param coef_vcov A numeric matrix of estimated covariances between coefficients.
+#' @param y,x,distr,param,scaling,regress,p,q,par_static,par_link,par_init,coef_fix_value,coef_fix_other,coef_fix_special,coef_bound_lower,coef_bound_upper,coef_est,coef_vcov When \code{gas_object} is not supplied, the estimated model can be specified using these individual arguments. See the arguments and value of the \code{\link[gasmodel:gas]{gas()}} function for more details.
 #'
 #' @return A \code{list} of S3 class \code{gas_filter} with components:
 #' \item{data$y}{The time series.}
@@ -99,56 +99,22 @@ gas_filter <- function(gas_object = NULL, method = "simulated_coefs", coef_set =
   } else if (!is.null(gas_object)) {
     stop("Unsupported class of gas_object.")
   } else {
-    model <- list()
-    model$distr <- check_my_distr(distr = distr)
-    model$param <- check_my_param(param = param, distr = model$distr)
-    model$scaling <- check_my_scaling(scaling = scaling)
-    model$regress <- check_my_regress(regress = regress)
-    info_distr <- info_distribution(distr = model$distr, param = model$param)
-    data <- list()
-    data$y <- check_my_y(y = y, dim = info_distr$dim, type = info_distr$type)
-    model$t <- check_my_t(y = data$y)
-    model$t_ahead <- check_my_t(t = t_ahead, positive = FALSE)
-    model$n <- check_my_n(y = data$y)
-    info_par <- info_parameters(distr = model$distr, param = model$param, n = model$n)
-    data$x <- check_my_x(x = x, t = model$t, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    model$m <- check_my_m(x = data$x)
-    data$x_ahead <- check_my_x(x = x_ahead, t = model$t_ahead, m = model$m, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    model$p <- check_my_p(p = p, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    model$q <- check_my_q(q = q, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    model$par_static <- check_my_par_static(par_static = par_static, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    model$par_static[model$m == 0L & model$p == 0L & model$q == 0L] <- TRUE
-    data$x[model$par_static] <- list(matrix(NA_real_, nrow = model$t, ncol = 0L))
-    model$m[model$par_static] <- 0L
-    model$p[model$par_static] <- 0L
-    model$q[model$par_static] <- 0L
-    model$par_link <- check_my_par_link(par_link = par_link, par_static = model$par_static, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    info_par <- info_linked_parameters(info_par = info_par, par_link = model$par_link)
-    model$m <- name_vector(model$m, info_par$par_names)
-    model$p <- name_vector(model$p, info_par$par_names)
-    model$q <- name_vector(model$q, info_par$par_names)
-    model$par_static <- name_vector(model$par_static, info_par$par_names)
-    model$par_link <- name_vector(model$par_link, info_par$par_names)
-    model$par_init <- name_vector(check_my_par_init(par_init = par_init, par_num = info_par$par_num), info_par$par_names)
-    info_coef <- info_coefficients(m = model$m, p = model$p, q = model$q, par_static = model$par_static, par_names = info_par$par_names, par_num = info_par$par_num, group_names = info_par$group_names, group_of_par_names = info_par$group_of_par_names)
-    model$coef_fix_value <- check_my_coef_fix_value(coef_fix_value = coef_fix_value, coef_num = info_coef$coef_num)
-    model$coef_fix_other <- check_my_coef_fix_other(coef_fix_other = coef_fix_other, coef_fix_value = model$coef_fix_value, coef_num = info_coef$coef_num)
-    model$coef_fix_special <- check_my_coef_fix_special(coef_fix_special = coef_fix_special)
-    model[c("coef_fix_value", "coef_fix_other")] <- fixed_coefficients(model = model, info_par = info_par, info_coef = info_coef)
-    model$coef_bound_lower <- name_vector(check_my_coef_bound_lower(coef_bound_lower = coef_bound_lower, par_static = model$par_static, par_support = info_par$par_support, par_num = info_par$par_num, coef_in_par_num = info_coef$coef_in_par_num, coef_num = info_coef$coef_num), info_coef$coef_names)
-    model$coef_bound_upper <- name_vector(check_my_coef_bound_upper(coef_bound_upper = coef_bound_upper, coef_bound_lower = model$coef_bound_lower, par_static = model$par_static, par_support = info_par$par_support, par_num = info_par$par_num, coef_in_par_num = info_coef$coef_in_par_num, coef_num = info_coef$coef_num), info_coef$coef_names)
-    info_theta <- info_thetas(coef_fix_value = model$coef_fix_value, coef_fix_other = model$coef_fix_other, coef_names = info_coef$coef_names)
-    fun <- list()
-    fun$score <- setup_fun_score(distr = model$distr, param = model$param, scaling = model$scaling, orthog = info_distr$orthog, par_trans = info_par$par_trans, par_static = model$par_static)
-    fun$random <- setup_fun_random(distr = model$distr, param = model$param, par_trans = info_par$par_trans)
-    filter <- list()
-    filter$method <- check_my_method(method = method, values = c("given_coefs", "simulated_coefs"))
-    comp <- list()
-    comp$rep_ahead <- check_generic_positive_integer_scalar(arg = rep_ahead, arg_name = "rep_ahead")
-    comp$quant <- check_generic_probability_vector(arg = quant, arg_name = "quant")
+    # Load auxiliary variables:
+    load <- load_filter(method = method, coef_set = coef_set, rep_gen = rep_gen, t_ahead = t_ahead, x_ahead = x_ahead, rep_ahead = rep_ahead, quant = quant, y = y, x = x, distr = distr, param = param, scaling = scaling, regress = regress, p = p, q = q, par_static = par_static, par_link = par_link, par_init = par_init, coef_fix_value = coef_fix_value, coef_fix_other = coef_fix_other, coef_fix_special = coef_fix_special, coef_bound_lower = coef_bound_lower, coef_bound_upper = coef_bound_upper, coef_est = coef_est, coef_vcov = coef_vcov)
+    data <- load$data
+    model <- load$model
+    fun <- load$fun
+    info_distr <- load$info_distr
+    info_par <- load$info_par
+    info_coef <- load$info_coef
+    info_theta <- load$info_theta
+    comp <- load$comp
+    filter <- load$filter
+    # Prepare coefficients from supplied set:
     if (filter$method == "given_coefs") {
       model$coef_set <- name_matrix(check_my_coef_set(coef_set = coef_set, coef_num = info_coef$coef_num), paste0("coef", 1:nrow(coef_set)), info_coef$coef_names)
       comp$rep_gen <- nrow(model$coef_set)
+    # Simulate coefficients:
     } else if (method == "simulated_coefs") {
       comp$rep_gen <- check_generic_positive_integer_scalar(arg = rep_gen, arg_name = "rep_gen")
       comp$coef_est <- check_my_coef_est(coef_est = coef_est, coef_num = info_coef$coef_num)
@@ -163,6 +129,7 @@ gas_filter <- function(gas_object = NULL, method = "simulated_coefs", coef_set =
         model$coef_set[i, ] <- convert_theta_vector_to_coef_vector(pmax(pmin(comp$theta_set[i, ], comp$theta_bound_upper), comp$theta_bound_lower), coef_fix_value = model$coef_fix_value, coef_fix_other = model$coef_fix_other)
       }
     }
+    # Initialize paths:
     comp$pre_num <- max(c(model$p, model$q, 1L))
     comp$full_num <- comp$pre_num + model$t + model$t_ahead
     comp$average_x <- lapply(1:info_par$par_num, function(i) { colMeans(data$x[[i]], na.rm = TRUE) })
@@ -179,12 +146,14 @@ gas_filter <- function(gas_object = NULL, method = "simulated_coefs", coef_set =
     comp$par_tv_ahead_path <- array(NA_real_, dim = c(comp$rep_gen * comp$rep_ahead, model$t_ahead, info_par$par_num))
     comp$score_tv_path <- array(NA_real_, dim = c(comp$rep_gen, model$t, info_par$par_num))
     comp$score_tv_ahead_path <- array(NA_real_, dim = c(comp$rep_gen * comp$rep_ahead, model$t_ahead, info_par$par_num))
+    # Loop through bootstrap samples:
     for (b in 1:comp$rep_gen) {
       comp$struc <- convert_coef_vector_to_struc_list(coef_vec = model$coef_set[b, ], m = model$m, p = model$p, q = model$q, par_names = info_par$par_names, par_of_coef_names = info_coef$par_of_coef_names)
       comp$omega_vector <- sapply(comp$struc, function(e) { e$omega })
       comp$beta_list <- lapply(comp$struc, function(e) { e$beta })
       comp$alpha_list <- lapply(comp$struc, function(e) { e$alpha })
       comp$phi_list <- lapply(comp$struc, function(e) { e$phi })
+      # Compute path for static model:
       if (all(model$p + model$q == 0L)) {
         comp$par_init <- model$par_init
         if (any(is.na(comp$par_init))) {
@@ -217,6 +186,7 @@ gas_filter <- function(gas_object = NULL, method = "simulated_coefs", coef_set =
             comp$score_tv_ahead_path[(b - 1L) * comp$rep_ahead + a, , ] <- comp$score_tv[(comp$pre_num + model$t + 1L):comp$full_num, ]
           }
         }
+      # Compute path for dynamic joint model:
       } else if (model$regress == "joint") {
         comp$par_init <- model$par_init
         if (any(is.na(comp$par_init))) {
@@ -259,6 +229,7 @@ gas_filter <- function(gas_object = NULL, method = "simulated_coefs", coef_set =
             comp$score_tv_ahead_path[(b - 1L) * comp$rep_ahead + a, , ] <- comp$score_tv[(comp$pre_num + model$t + 1L):comp$full_num, ]
           }
         }
+      # Compute path for dynamic separate model:
       } else if (model$regress == "sep") {
         comp$err_tv <- matrix(NA_real_, nrow = comp$full_num, ncol = info_par$par_num)
         comp$err_init <- model$par_init
@@ -311,6 +282,7 @@ gas_filter <- function(gas_object = NULL, method = "simulated_coefs", coef_set =
       comp$par_tv_path[b, , ] <- comp$par_tv[(comp$pre_num + 1L):(comp$pre_num + model$t), ]
       comp$score_tv_path[b, , ] <- comp$score_tv[(comp$pre_num + 1L):(comp$pre_num + model$t), ]
     }
+    # Format results:
     info_data <- info_data(y = data$y, x = data$x)
     data$y <- name_matrix(data$y, info_data$index_time, info_data$index_series, drop = c(FALSE, TRUE))
     data$x <- name_list_of_matrices(data$x, info_par$par_names, info_data$index_time_list, info_data$index_vars_list, drop = c(FALSE, TRUE), zero = c(FALSE, TRUE))
@@ -336,6 +308,7 @@ gas_filter <- function(gas_object = NULL, method = "simulated_coefs", coef_set =
       data$x_ahead <- NULL
       model$t_ahead <- NULL
     }
+    # Report results:
     report <- list(data = data, model = model, filter = filter)
     class(report) <- "gas_filter"
     return(report)
