@@ -11,14 +11,13 @@
 #' Method \code{"simulated_paths"} repeatedly simulates time series, simultaneously filters time-varying parameters, and then estimates mean, standard deviation, and quantiles (see Blasques et al., 2016).
 #' Instead of supplying arguments about the model, the function can be applied to the \code{gas} object obtained by the \code{\link[gasmodel:gas]{gas()}} function.
 #'
-#' @inheritParams gas
-#' @inheritParams gas_simulate
-#'
+#' @param gas_object An optional GAS estimate, i.e. a list of S3 class \code{gas} returned by function \code{\link[gasmodel:gas]{gas()}}.
 #' @param method  A method used for forecasting. Supported methods are \code{"mean_path"} and \code{"simulated_paths"}.
 #' @param t_ahead A number of observations to forecast.
 #' @param x_ahead Out-of-sample exogenous variables. For a single variable common for all time-varying parameters, a numeric vector. For multiple variables common for all time-varying parameters, a numeric matrix with observations in rows. For individual variables for each time-varying parameter, a list of numeric vectors or matrices in the above form. The number of observation must be equal to \code{t_ahead}.
 #' @param rep_ahead A number of simulation repetitions for \code{method = "simulated_paths"}.
 #' @param quant A numeric vector of probabilities determining quantiles for \code{method = "simulated_paths"}.
+#' @param y,x,distr,param,scaling,regress,p,q,par_static,par_link,par_init,coef_est When \code{gas_object} is not supplied, the estimated model can be specified using these individual arguments. See the arguments and value of the \code{\link[gasmodel:gas]{gas()}} function for more details.
 #'
 #' @return A \code{list} of S3 class \code{gas_forecast} with components:
 #' \item{data$y}{The time series.}
@@ -87,63 +86,19 @@ gas_forecast <- function(gas_object = NULL, method = "mean_path", t_ahead = 1L,
   } else if (!is.null(gas_object)) {
     stop("Unsupported class of gas_object.")
   } else {
-    model <- list()
-    model$distr <- check_my_distr(distr = distr)
-    model$param <- check_my_param(param = param, distr = model$distr)
-    model$scaling <- check_my_scaling(scaling = scaling)
-    model$regress <- check_my_regress(regress = regress)
-    info_distr <- info_distribution(distr = model$distr, param = model$param)
-    data <- list()
-    data$y <- check_my_y(y = y, dim = info_distr$dim, type = info_distr$type)
-    model$t <- check_my_t(y = data$y)
-    model$t_ahead <- check_my_t(t = t_ahead)
-    model$n <- check_my_n(y = data$y)
-    info_par <- info_parameters(distr = model$distr, param = model$param, n = model$n)
-    data$x <- check_my_x(x = x, t = model$t, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    model$m <- check_my_m(x = data$x)
-    data$x_ahead <- check_my_x(x = x_ahead, t = model$t_ahead, m = model$m, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    model$p <- check_my_p(p = p, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    model$q <- check_my_q(q = q, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    model$par_static <- check_my_par_static(par_static = par_static, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    model$par_static[model$m == 0L & model$p == 0L & model$q == 0L] <- TRUE
-    data$x[model$par_static] <- list(matrix(NA_real_, nrow = model$t, ncol = 0L))
-    model$m[model$par_static] <- 0L
-    model$p[model$par_static] <- 0L
-    model$q[model$par_static] <- 0L
-    model$par_link <- check_my_par_link(par_link = par_link, par_static = model$par_static, par_num = info_par$par_num, group_num = info_par$group_num, par_in_group_num = info_par$par_in_group_num)
-    info_par <- info_linked_parameters(info_par = info_par, par_link = model$par_link)
-    model$m <- name_vector(model$m, info_par$par_names)
-    model$p <- name_vector(model$p, info_par$par_names)
-    model$q <- name_vector(model$q, info_par$par_names)
-    model$par_static <- name_vector(model$par_static, info_par$par_names)
-    model$par_link <- name_vector(model$par_link, info_par$par_names)
-    model$par_init <- name_vector(check_my_par_init(par_init = par_init, par_num = info_par$par_num), info_par$par_names)
-    info_coef <- info_coefficients(m = model$m, p = model$p, q = model$q, par_static = model$par_static, par_names = info_par$par_names, par_num = info_par$par_num, group_names = info_par$group_names, group_of_par_names = info_par$group_of_par_names)
-    model$coef_est <- name_vector(check_my_coef_est(coef_est = coef_est, coef_num = info_coef$coef_num), info_coef$coef_names)
-    fun <- list()
-    fun$mean <- setup_fun_mean(distr = model$distr, param = model$param, par_trans = info_par$par_trans)
-    fun$score <- setup_fun_score(distr = model$distr, param = model$param, scaling = model$scaling, orthog = info_distr$orthog, par_trans = info_par$par_trans, par_static = model$par_static)
-    fun$random <- setup_fun_random(distr = model$distr, param = model$param, par_trans = info_par$par_trans)
-    comp <- list()
-    comp$pre_num <- max(c(model$p, model$q, 1L))
-    comp$full_num <- comp$pre_num + model$t + model$t_ahead
-    comp$average_x <- lapply(1:info_par$par_num, function(i) { colMeans(data$x[[i]], na.rm = TRUE) })
-    comp$y <- rbind(matrix(NA_real_, nrow = comp$pre_num, ncol = model$n), data$y, matrix(NA_real_, nrow = model$t_ahead, ncol = model$n))
-    comp$x <- lapply(1:info_par$par_num, function(i) { rbind(matrix(NA_real_, nrow = comp$pre_num, ncol = model$m[i]), data$x[[i]], data$x_ahead[[i]]) })
-    comp$par_tv <- matrix(NA_real_, nrow = comp$full_num, ncol = info_par$par_num)
-    comp$score_tv <- matrix(NA_real_, nrow = comp$full_num, ncol = info_par$par_num)
-    comp$idx_na <- which(rowSums(cbind(c(rowSums(is.na(comp$y))[1L:(comp$pre_num + model$t)], rep(0L, model$t_ahead)), sapply(comp$x, function(e) { rowSums(is.na(e)) }))) > 0L)
-    comp$idx_ok <- which(!((1:comp$full_num) %in% comp$idx_na))
-    comp$idx_ok_regular <- comp$idx_ok[comp$idx_ok <= comp$pre_num + model$t]
-    comp$idx_ok_ahead <- comp$idx_ok[comp$idx_ok > comp$pre_num + model$t]
-    comp$struc <- convert_coef_vector_to_struc_list(coef_vec = model$coef_est, m = model$m, p = model$p, q = model$q, par_names = info_par$par_names, par_of_coef_names = info_coef$par_of_coef_names)
-    comp$omega_vector <- sapply(comp$struc, function(e) { e$omega })
-    comp$beta_list <- lapply(comp$struc, function(e) { e$beta })
-    comp$alpha_list <- lapply(comp$struc, function(e) { e$alpha })
-    comp$phi_list <- lapply(comp$struc, function(e) { e$phi })
-    forecast <- list()
-    forecast$method <- check_my_method(method = method, values = c("mean_path", "simulated_paths"))
+    # Load auxiliary variables:
+    load <- load_forecast(method = method, t_ahead = t_ahead, x_ahead = x_ahead, rep_ahead = rep_ahead, quant = quant, y = y, x = x, distr = distr, param = param, scaling = scaling, regress = regress, p = p, q = q, par_static = par_static, par_link = par_link, par_init = par_init, coef_est = coef_est)
+    data <- load$data
+    model <- load$model
+    fun <- load$fun
+    info_distr <- load$info_distr
+    info_par <- load$info_par
+    info_coef <- load$info_coef
+    comp <- load$comp
+    forecast <- load$forecast
+    # Forecast using mean path method:
     if (forecast$method == "mean_path") {
+      # Compute path for static model:
       if (all(model$p + model$q == 0L)) {
         comp$par_init <- model$par_init
         if (any(is.na(comp$par_init))) {
@@ -162,6 +117,7 @@ gas_forecast <- function(gas_object = NULL, method = "mean_path", t_ahead = 1L,
           comp$score_tv[j, ] <- fun$score(y = comp$y[j, , drop = FALSE], f = comp$par_tv[j, , drop = FALSE])
         }
         comp$score_tv[comp$idx_ok_ahead, ] <- 0
+      # Compute path for dynamic joint model:
       } else if (model$regress == "joint") {
         comp$par_init <- model$par_init
         if (any(is.na(comp$par_init))) {
@@ -191,6 +147,7 @@ gas_forecast <- function(gas_object = NULL, method = "mean_path", t_ahead = 1L,
           }
           comp$par_tv[j, ] <- comp$par_tv[j, ] + cur_e
         }
+      # Compute path for dynamic separate model:
       } else if (model$regress == "sep") {
         comp$err_tv <- matrix(NA_real_, nrow = comp$full_num, ncol = info_par$par_num)
         comp$err_init <- model$par_init
@@ -227,6 +184,7 @@ gas_forecast <- function(gas_object = NULL, method = "mean_path", t_ahead = 1L,
         }
         comp$par_tv[comp$idx_ok_ahead, ] <- comp$par_tv[comp$idx_ok_ahead, ] + comp$err_tv[comp$idx_ok_ahead, ]
       }
+      # Format results:
       comp$y[comp$idx_ok_ahead, ] <- fun$mean(f = comp$par_tv[comp$idx_ok_ahead, , drop = FALSE])
       info_data <- info_data(y = data$y, x = data$x)
       data$y <- name_matrix(data$y, info_data$index_time, info_data$index_series, drop = c(FALSE, TRUE))
@@ -236,12 +194,14 @@ gas_forecast <- function(gas_object = NULL, method = "mean_path", t_ahead = 1L,
       forecast$y_ahead_mean <- name_matrix(comp$y[(comp$pre_num + model$t + 1L):comp$full_num, , drop = FALSE], info_data_ahead$index_time, info_data_ahead$index_series, drop = c(FALSE, TRUE))
       forecast$par_tv_ahead_mean <- name_matrix(comp$par_tv[(comp$pre_num + model$t + 1L):comp$full_num, , drop = FALSE], info_data_ahead$index_time, info_par$par_names, drop = c(FALSE, TRUE))
       forecast$score_tv_ahead_mean <- name_matrix(comp$score_tv[(comp$pre_num + model$t + 1L):comp$full_num, , drop = FALSE], info_data_ahead$index_time, info_par$par_names, drop = c(FALSE, TRUE))
+    # Forecast using simulated path method:
     } else if (forecast$method == "simulated_paths") {
       comp$rep_ahead <- check_generic_positive_integer_scalar(arg = rep_ahead, arg_name = "rep_ahead")
       comp$quant <- check_generic_probability_vector(arg = quant, arg_name = "quant")
       comp$y_ahead_path <- array(NA_real_, dim = c(comp$rep_ahead, model$t_ahead, model$n))
       comp$par_tv_ahead_path <- array(NA_real_, dim = c(comp$rep_ahead, model$t_ahead, info_par$par_num))
       comp$score_tv_ahead_path <- array(NA_real_, dim = c(comp$rep_ahead, model$t_ahead, info_par$par_num))
+      # Compute path for static model:
       if (all(model$p + model$q == 0L)) {
         comp$par_init <- model$par_init
         if (any(is.na(comp$par_init))) {
@@ -268,6 +228,7 @@ gas_forecast <- function(gas_object = NULL, method = "mean_path", t_ahead = 1L,
           comp$par_tv_ahead_path[a, , ] <- comp$par_tv[(comp$pre_num + model$t + 1L):comp$full_num, ]
           comp$score_tv_ahead_path[a, , ] <- comp$score_tv[(comp$pre_num + model$t + 1L):comp$full_num, ]
         }
+      # Compute path for dynamic joint model:
       } else if (model$regress == "joint") {
         comp$par_init <- model$par_init
         if (any(is.na(comp$par_init))) {
@@ -308,6 +269,7 @@ gas_forecast <- function(gas_object = NULL, method = "mean_path", t_ahead = 1L,
           comp$par_tv_ahead_path[a, , ] <- comp$par_tv[(comp$pre_num + model$t + 1L):comp$full_num, ]
           comp$score_tv_ahead_path[a, , ] <- comp$score_tv[(comp$pre_num + model$t + 1L):comp$full_num, ]
         }
+      # Compute path for dynamic separate model:
       } else if (model$regress == "sep") {
         comp$err_tv <- matrix(NA_real_, nrow = comp$full_num, ncol = info_par$par_num)
         comp$err_init <- model$par_init
@@ -355,6 +317,7 @@ gas_forecast <- function(gas_object = NULL, method = "mean_path", t_ahead = 1L,
           comp$score_tv_ahead_path[a, , ] <- comp$score_tv[(comp$pre_num + model$t + 1L):comp$full_num, ]
         }
       }
+      # Format results:
       info_data <- info_data(y = data$y, x = data$x)
       data$y <- name_matrix(data$y, info_data$index_time, info_data$index_series, drop = c(FALSE, TRUE))
       data$x <- name_list_of_matrices(data$x, info_par$par_names, info_data$index_time_list, info_data$index_vars_list, drop = c(FALSE, TRUE), zero = c(FALSE, TRUE))
@@ -370,6 +333,7 @@ gas_forecast <- function(gas_object = NULL, method = "mean_path", t_ahead = 1L,
       forecast$score_tv_ahead_sd <- name_matrix(apply(comp$score_tv_ahead_path, 2:3, stats::sd, na.rm = TRUE), info_data_ahead$index_time, info_par$par_names, drop = c(FALSE, TRUE))
       forecast$score_tv_ahead_quant <- name_3d_array(aperm(array(apply(comp$score_tv_ahead_path, 2:3, stats::quantile, probs = comp$quant, na.rm = TRUE), dim = c(length(comp$quant), model$t_ahead, info_par$par_num)), c(2, 3, 1)), info_data_ahead$index_time, info_par$par_names, paste0(comp$quant * 100, "%"), drop = c(FALSE, TRUE, TRUE), zero = c(FALSE, FALSE, TRUE))
     }
+    # Report results
     report <- list(data = data, model = model, forecast = forecast)
     class(report) <- "gas_forecast"
     return(report)
